from __future__ import annotations

import os
from typing import Any, Dict, Optional

from dotenv import load_dotenv
from mcp.server.fastmcp import FastMCP

from .flink_sql_gateway_client import FlinkSqlGatewayClient

def build_server() -> FastMCP:
    load_dotenv()

    server = FastMCP("Flink SQLGateway MCP Server")

    client = FlinkSqlGatewayClient(os.getenv("SQL_GATEWAY_API_BASE_URL"))

    @server.resource("https://mcp.local/flink/info")
    def flink_info() -> Dict[str, Any]:
        """Get Flink cluster info via SQL Gateway /v1/info."""
        return client.get_info()

    @server.tool()
    def open_new_session(properties: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Open a new SQL Gateway session. Return includes 'sessionHandle'.
        If your agent/tooling supports memory, persist the handle for reuse.
        """
        return client.open_session(properties)

    @server.tool()
    def get_session_handle_config(session_handle: str) -> Dict[str, Any]:
        """Get details/configuration for a session handle."""
        return client.get_session(session_handle)

    @server.tool()
    def execute_query(
        session_handle: str,
        query: str,
        execution_config: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """
        Execute a SQL statement in the given session. Returns an object that includes 'operationHandle'.
        Use the operation handle to poll status and fetch results.
        """
        return client.execute_statement(session_handle, query, execution_config=execution_config)

    @server.tool()
    def get_operation_status(session_handle: str, operation_handle: str) -> Dict[str, Any]:
        """Get status of an operation (e.g., RUNNING/FINISHED/ERROR)."""
        return client.get_operation_status(session_handle, operation_handle)

    @server.tool()
    def fetch_result_page(
        session_handle: str,
        operation_handle: str,
        token: int = 0,
    ) -> Dict[str, Any]:
        """
        Fetch paginated results for an operation. Start with token=0 and increment until 'isEnd' or similar field indicates completion.
        """
        return client.fetch_result(session_handle, operation_handle, token=token)

    @server.prompt()
    def manage_session_handle() -> str:
        return (
            "If you do not have a valid 'sessionHandle', call open_new_session() first and remember the handle. "
            "If a session has expired or is invalid, create a new one and continue."
        )

    return server


def main() -> None:
    server = build_server()
    server.run(transport="stdio")


if __name__ == "__main__":
    main()







import os
import time
from typing import Callable

import httpx
import pytest

from flink_mcp.flink_sql_gateway_client import FlinkSqlGatewayClient


# ----------------------
# Unit tests (mocked IO)
# ----------------------

def _make_mock_client(responder: Callable[[httpx.Request], httpx.Response]) -> httpx.Client:
    transport = httpx.MockTransport(responder)
    return httpx.Client(transport=transport)


def test_get_info_mocked() -> None:
    def responder(request: httpx.Request) -> httpx.Response:
        assert request.method == "GET"
        assert request.url.path == "/v1/info"
        return httpx.Response(200, json={"productName": "Apache Flink", "version": "test"})

    client = FlinkSqlGatewayClient(base_url="http://mock", client=_make_mock_client(responder))
    info = client.get_info()
    assert isinstance(info, dict)
    assert info.get("version") == "test"


def test_statement_flow_mocked() -> None:
    session_handle = "session-123"
    operation_handle = "op-456"

    def responder(request: httpx.Request) -> httpx.Response:
        # Create session
        if request.method == "POST" and request.url.path == "/v3/sessions":
            return httpx.Response(200, json={"sessionHandle": session_handle, "properties": {}})

        # Submit statement
        if request.method == "POST" and request.url.path == f"/v3/sessions/{session_handle}/statements":
            body = request.content
            assert body and b"statement" in body
            return httpx.Response(200, json={"operationHandle": operation_handle})

        # Operation status
        if request.method == "GET" and request.url.path == f"/v3/sessions/{session_handle}/operations/{operation_handle}/status":
            return httpx.Response(200, json={"status": {"status": "FINISHED"}})

        # Fetch result page 0
        if (
            request.method == "GET"
            and request.url.path
            == f"/v3/sessions/{session_handle}/operations/{operation_handle}/result/0"
        ):
            # Optionally ensure rowFormat=JSON is requested
            assert b"rowFormat=JSON" in (request.url.query or b"")
            return httpx.Response(200, json={"result": "ok", "data": [[1]]})

        return httpx.Response(404, json={"message": "not mocked"})

    client = FlinkSqlGatewayClient(base_url="http://mock", client=_make_mock_client(responder))

    created = client.open_session()
    assert created.get("sessionHandle") == session_handle

    submitted = client.execute_statement(session_handle, "SELECT 1")
    assert submitted.get("operationHandle") == operation_handle

    status = client.get_operation_status(session_handle, operation_handle)
    assert status.get("status", {}).get("status") == "FINISHED"

    result = client.fetch_result(session_handle, operation_handle, token=0)
    assert result.get("result") == "ok"


# ---------------------------------
# Live integration tests (real HTTP)
# ---------------------------------
def _get_base_url() -> str:
    return os.getenv("SQL_GATEWAY_API_BASE_URL", "http://localhost:8083")


@pytest.mark.integration
def test_live_info() -> None:
    base_url = _get_base_url()
    client = FlinkSqlGatewayClient(base_url=base_url)
    try:
        info = client.get_info()
        assert isinstance(info, dict)
        assert "version" in info  # Flink typically exposes version here
    finally:
        client.close()


@pytest.mark.integration
def test_live_open_session_minimal() -> None:
    base_url = _get_base_url()
    client = FlinkSqlGatewayClient(base_url=base_url)
    try:
        created = client.open_session()
        assert isinstance(created, dict)
        # Be lenient about schema across versions; just require a non-empty dict
        assert len(created) > 0
    finally:
        client.close()


@pytest.mark.integration
def test_live_select_one() -> None:
    base_url = _get_base_url()
    client = FlinkSqlGatewayClient(base_url=base_url)
    try:
        created = client.open_session()
        assert isinstance(created, dict) and len(created) > 0
        session_handle = created.get("sessionHandle")
        if isinstance(session_handle, dict):
            session_handle = session_handle.get("identifier") or session_handle.get("id") or session_handle.get("sessionId")
        assert isinstance(session_handle, str) and session_handle

        submitted = client.execute_statement(session_handle, "SELECT 1")
        operation_handle = submitted.get("operationHandle")
        if isinstance(operation_handle, dict):
            operation_handle = operation_handle.get("identifier") or operation_handle.get("id") or operation_handle.get("operationId")
        assert isinstance(operation_handle, str) and operation_handle

        deadline = time.monotonic() + 15.0
        status_value = None
        while time.monotonic() < deadline:
            status_resp = client.get_operation_status(session_handle, operation_handle)
            status_value = status_resp["status"]
            if status_value in {"FINISHED", "CANCELED", "FAILED"}:
                break
            time.sleep(0.2)

        assert status_value == "FINISHED"

        # Try first three pages quickly; SELECT 1 should appear on the first or second page
        rows = []
        for token in (0, 1, 2):
            resp = client.fetch_result(session_handle, operation_handle, token=token)
            data = resp["results"]["data"]
            if data:
                rows = data
                break
            time.sleep(0.1)

        assert rows, "No rows returned from result pages 0..2"
        first = rows[0]
        fields = first["fields"]
        assert fields[0] == 1
    finally:
        client.close()


